from decimal import Decimal
from typing import List, Optional
from pydantic import BaseModel, Field, AnyUrl
from .codes import EncryptionMethod, HashMethod


class IdentifierScheme(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#identifierscheme
    type: str = "IdentifierScheme"

    id: AnyUrl  # from vocabulary.uncefact.org/identifierSchemes
    name: str


class Party(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#Party
    type: str = "Party"

    id: AnyUrl
    name: str
    registeredId: Optional[str] = None
    idScheme: Optional[IdentifierScheme] = None
    # description: str in jargon, but not in context
    registrationCountry: Optional[str] = None
    organizationWebsite: Optional[str] = None
    industryCategory: Optional[str] = None
    otherIdentiifer: Optional[str] = None

class Identifier(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#identifier
    type: str = "Identifier"

    id: AnyUrl
    name: str
    registeredId: Optional[str] = None
    idScheme: Optional[IdentifierScheme] = None


class BinaryFile(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#binaryfile
    type: str = "BinaryFile"

    fileName: str
    fileType: str  # https://mimetype.io/all-types
    file: str  #Base64


class Link(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#link
    type: str = "Link"

    linkURL: AnyUrl
    linkName: str
    linkType: str  # drawn from a controlled vocabulary


class SecureLink(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#securelink
    type: str = "SecureLink"

    linkUrl: AnyUrl
    linkName: str
    linkType: str
    hashDigest: str
    hashMethod: HashMethod
    encryptionMethod: EncryptionMethod


class Measure(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#measure
    type: str = "Measure"

    value: float
    unit: str = Field(
        max_length="3")  # from https://vocabulary.uncefact.org/UnitMeasureCode


class Endorsement(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#endorsement
    type: str = "Endorsement"

    id: AnyUrl
    name: str
    trustmark: Optional[BinaryFile] = None
    issuingAuthority: Identifier
    accreditationCertification: Optional[Link] = None


class Point3D(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#point3d
    type: str = "Point3D"
    data: List[Decimal]

class Point(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#point
    type: str = "Point" 

    coordinates: Point3D


class Point3DWrapper(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#point3dwrapper
    type: str = "Point3DWrapper"

    data: Point3D

class Polygon(BaseModel): 
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#polygon
    type: str = "Polygon" 

    coordinates: Point3DWrapper


class LocationInformation(BaseModel): 
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#location
    type: str = "LocationInformation"

    plusCode: Optional[AnyUrl] = None
    geoLocation: Optional[Point] = None
    geoBoundary: Optional[Polygon] = None

class Address(BaseModel): 
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#address
    type:str = "Address"

    streetAddress: str
    postalCode: str
    addressLocality: str #city/suburb
    addressRegion: str #state/province
    addressCountry: str # ISO-3166 two letter country code. https://vocabulary.uncefact.org/CountryId


from datetime import date
from decimal import Decimal
from typing import List, Optional, Union
from pydantic import BaseModel, AnyUrl, Field
from .codes import AssessorLevelCode, AssessmentLevelCode, AttestationType, ConformityTopicCode
from .base import Identifier, Measure, BinaryFile, SecureLink, Endorsement, IdentifierScheme, Party, LocationInformation, Address


class Standard(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#standard
    type: str = "Standard"

    id: AnyUrl
    name: str
    issuingParty: Identifier
    issueDate: str  #iso8601 datetime string


class Regulation(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#regulation
    type: str = "Regulation"

    id: AnyUrl
    name: str
    jurisdictionCountry: str  #countryCode from https://vocabulary.uncefact.org/CountryId
    administeredBy: Identifier
    effectiveDate: str  #iso8601 datetime string


class Metric(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#metric
    type: str = "Metric"

    metricName: str
    metricValue: Measure
    score: Optional[str] = None
    accuracy: Decimal


class Criterion(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#criterion
    type: str = "Criterion"

    id: AnyUrl
    name: str
    thresholdValues: List[Metric]


class Facility(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#facility
    type: str = "Facility"

    id: Optional[AnyUrl] # The globally unique ID of the entity as a resolvable URL according to ISO 18975.
    name: str
    registeredId: Optional[str] = None
    idScheme: Optional[IdentifierScheme] = None

    description: Optional[str] = None
    countryOfOpertation:  Optional[str] = None
    processCategory: Optional[str] = None
    operatedByParty: Optional[bool] = None
    otherIdentifier: Optional[str] = None

    #jargon makes it look like a class, schema looks like an attribute. 
    locationInformation: Optional[Union[str, LocationInformation]] = None 
    address: Optional[Address] = None

    IDverifiedByCAB: bool


class Product(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#product
    type: str = "Product"

    id: AnyUrl  # The globally unique ID of the entity as a resolvable URL according to ISO 18975.
    name: str
    registeredId: Optional[str] = None
    idScheme: Optional[IdentifierScheme] = None
    serialNumber: Optional[str] = None
    batchNumber: Optional[str] = None
    productImage: Optional[bytes] = None
    description: Optional[str] = None
    productCategory: Optional[str] = None
    furtherInformation: Optional[str] = None
    producedbyParty: Optional[bool] = None
    producedatFacility: Optional[bool] = None
    dimensions: Optional[str] = None
    productionDate: Optional[str] = None
    countryOfProduction: Optional[str] = None
    IDverifiedByCAB: Optional[bool] = None


class ConformityAssessment(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#conformityassessment
    type: str = "ConformityAssessment"

    id: AnyUrl
    assessmentDate: date

    referenceStandard: Optional[Standard] = None  #defines the specification
    referenceRegulation: Optional[Regulation] = None  #defines the regulation
    assessmentCriterion: Optional[List[Criterion]] = None  #defines the criteria
    declaredValues: Optional[List[Metric]] = None
    conformance: Optional[bool] = None
    conformityTopic: ConformityTopicCode

    assessedProduct: Optional[List[Product]] = None
    assessedFacility: Optional[List[Facility]] = None
    assessedOrganization: Optional[Party] = None
    auditor: Optional[Party] = None


class ConformityAssessmentScheme(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#conformityassessmentscheme
    type: str = "ConformityAssessmentScheme"

    id: str
    name: str
    issuingParty: Optional[Identifier] = None
    issueDate: Optional[str] = None  #ISO8601 datetime string
    trustmark: Optional[BinaryFile] = None


class ConformityAttestation(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#ConformityAttestation
    type: str = "ConformityAttestation"

    id: str
    name: str
    assessorLevel: Optional[AssessorLevelCode] = None
    assessmentLevel: AssessmentLevelCode
    attestationType: AttestationType
    description: Optional[str] = None  #missing from context file
    issuedToParty: Party
    authorisation: Optional[List[Endorsement]] = None
    conformityCertificate: Optional[SecureLink] = None
    auditableEvidence: Optional[SecureLink] = None
    scope: ConformityAssessmentScheme
    assessment: List[ConformityAssessment]
                                                                      

class CredentialIssuer(BaseModel):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#credentialissuer
    type: str = "CredentialIssuer"

    id: AnyUrl
    name: str
    otherIdentifier: Identifier

class DigitalConformityCredential(BaseModel):
    #https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#digitalconformitycredential
    context: str = Field(alias="@context")
    id: AnyUrl
    issuer: CredentialIssuer
    validFrom: str #DateTime
    validUntil: str #DateTime
    credentialSubject: ConformityAttestation


from enum import Enum


class AssessorLevelCode(str, Enum):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#assessorLevelCode
    Self = "Self"
    Commercial = "Commercial"
    Buyer = "Buyer"
    Membership = "Membership"
    Unspecified = "Unspecified"
    ThirdParty = "3rdParty"


class AssessmentLevelCode(str, Enum):
    #https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#assessmentlevelcode
    GovtApproval = "GovtApproval"
    GlobalMRA = "GlobalMRA"
    Accredited = "Accredited"
    Verified = "Verified"
    Validated = "Validated"
    Unspecified = "Unspecified"


class AttestationType(str, Enum):
    # https://uncefact.github.io/spec-untp/docs/specification/ConformityCredential/#attestationtype
    Certification = "Certification"
    Declaration = "Declaration"
    Inspection = "Inspection"
    Testing = "Testing"
    Verification = "Verification"
    Validation = "Validation"
    Calibration = "Calibration"


class HashMethod(str, Enum):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#hashmethodcode
    SHA256 = "SHA-256"
    SHA1 = "SHA-1"


class EncryptionMethod(str, Enum):
    NONE = "None"
    AES = "AES"


class ConformityTopicCode(str, Enum):
    # https://jargon.sh/user/unece/ConformityCredential/v/0.5.0/artefacts/readme/render#conformityTopicCode
    Environment_Energy = "Environment.Energy"
    Environment_Emissions = "Environment.Emissions"
    Environment_Water = "Environment.Water"
    Environment_Waste = "Environment.Waste"
    Environment_Deforestation = "Environment.Deforestation"
    Environment_Biodiversity = "Environment.Biodiversity"
    Cirularity_Content = "Circularity.Content"
    Cirularity_Design = "Circularity.Design"
    Social_Labour = "Social.Labour"
    Social_Rights = "Social.Rights"
    Social_Safety = "Social.Safety"
    Social_Community = "Social.Community"
    Governance_Ethics = "Governance.Ethics"
    Governance_Compliance = "Governance.Compliance"
    Governance_Transparency = "Governance.Transparency"
